;*********************************************************************
;* FileName     : SRAM_main.asm
;* Dependencies : See INCLUDES section below
;* Processor    : PIC16F family of PICmicro - 8 bit MCU      
;* Assembler/
;* Linker       : MPASM
;* Company      : Microchip Technology, Inc.
;*
;*                     Software License Agreement                      
;*                                                                     
;* Microchip Technology Inc. (Microchip) licenses this software to you
;* solely for use with Microchip microcontrollers and Microchip serial
;* EEPROM products.  The software is owned by Microchip and/or its
;* licensors, and is protected under applicable copyright laws.  All
;* rights reserved.
;*
;* SOFTWARE IS PROVIDED "AS IS."  MICROCHIP AND ITS LICENSOR EXPRESSLY
;* DISCLAIM ANY WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED,
;* INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
;* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
;* NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP AND ITS LICENSORS BE
;* LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
;* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
;* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
;* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
;* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
;*
;* Author                   Date                Version         Comment
;*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;* Deepak Kumar Rana        20-Feb-08           V1.0            Original
;*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    list        p=16f887      			; list directive to define processor
    #include    "p16f887.inc" 			; processor specific variable definitions
    #include    "SRAM_main.inc"


;******************CONFIGURATION BITS****************************************************
 __CONFIG    _CONFIG1, _LVP_OFF & _FCMEN_ON & _IESO_OFF & _BOR_OFF & _CPD_OFF & _CP_OFF & _MCLRE_ON & _PWRTE_ON & _WDT_OFF & _HS_OSC;
 __CONFIG    _CONFIG2, _WRT_OFF & _BOR21V

;**********************************************************************

	RESET_VECTOR    CODE    0x0000 		 ; processor reset vector
	pagesel start
	goto    start             			 ; go to beginning of program

	INT_VECTOR      CODE    0x0004 		 ; interrupt vector location


;write your ISR here
	
	retfie                  			; return from interrupt




;**************************************************************************
; Macro              :INIT


; Overview           :Macro gives the high address byte, low address byte ,data byte to be written
;					  to the SRAM	 
;						 
;                        
;*************************************************************
INIT   MACRO  ADRH ,ADRL,DAT   		; high address byte, low address byte ,data byte
       banksel Addr_High
       movlw  ADRH
       movwf  Addr_High		   		;high address byte of the SRAM location
       movlw  ADRL
       movwf  Addr_Low		  		;low address byte of the SRAM location	
       movlw  DAT
       movwf  Data_Out         		;data to be written to the SRAM location

       ENDM

MAIN_PROG       CODE

start:
        
     pagesel Init_SPI
     call  Init_SPI     			; intialize the MASSP module for SPI operation


;**************************************************************************  
 ;  Byte_Mode demo    
;**************************************************************************  
       banksel mode
       movlw  Byte_Mode  
       movwf  mode				   ; move the mode of the operation
       pagesel SRAMWriteStatusReg
       call  SRAMWriteStatusReg    ; write the mode to the status register of SRAM
        
       INIT  0x00,0x20,0xAA 	   ; Macro for the address location of SRAM and data to be written to 
        						   ; that location
       
       pagesel SRAMWriteByte
       call   SRAMWriteByte    	   ;write the data to the specified address location
  
       pagesel SRAMReadByte 
       call    SRAMReadByte    		  ;read the data from the specified address location

     
;**************************************************************************  
 ;  Page_Mode demo    
;**************************************************************************  
       banksel mode
       movlw  Page_Mode  
       movwf  mode           	 ; page mode of operation
       pagesel SRAMWriteStatusReg
       call  SRAMWriteStatusReg	 ; write the mode to the status register of SRAM

       INIT  0x00,0x20,0x44  	 ;Macro for the address location of SRAM and data to be written
       movlw  .32            	 ;Number of bytes to be written 
       movwf  count 

       pagesel Init_Datamem  	 ;Initialize the location in the controller data memory 
       call Init_Datamem  

       pagesel SRAMReadStatusReg ; Read the status register to confirm for the page mode of the SRAM
       call   SRAMReadStatusReg    
    
       banksel count        	 ; Number of bytes to be written to SRAM
       movlw  .32
       movwf  count 

       pagesel SRAMWritePage    	 ; call the write to page SRAM operation
       call    SRAMWritePage 

       banksel count        	 ; Number of bytes to be read from the SRAM 
       movlw  .32
       movwf  count 

       pagesel SRAMReadPage   		; Read the bytes from the SRAM
       call    SRAMReadPage
       
;**************************************************************************  
;   Sequential Mode demo    
;**************************************************************************  
       banksel mode
       movlw  Seq_Mode   	   ;sequential mode of operation
       movwf  mode

       pagesel SRAMWriteStatusReg
       call  SRAMWriteStatusReg  	   ; configure Status register of SRAM for sequentail mode
  
       pagesel SRAMReadStatusReg
       call  SRAMReadStatusReg  	   ; read Status register to confirm the operation
    
       INIT  0x00,0x10,0x77    ;Macro for the address location of SRAM and data to be written
       movlw  .40              ;number of bytes to written
       movwf  count
       
       pagesel Init_Datamem    ; Initialize the controller memory with the data to be written to SRAM
       call Init_Datamem  

       banksel count           ;Number of location/bytes to be written
       movlw  .40
       movwf  count

       pagesel SRAMWriteSeq 
       call    SRAMWriteSeq 	   ; perform the sequential write 

       banksel count
       movlw  .40      		   ;Number of locations to be read
       movwf  count


       pagesel SRAMReadSeq
       call    SRAMReadSeq 	   ; peform the sequential read from the SRAM
   
 
       	
        nop
       
        goto $

   


        END                      ; directive 'end of program'