
;*********************************************************************
;* FileName     : SRAM_Driver.asm
;* Dependencies : See INCLUDES section below
;* Processor    : PIC16F family of PICmicro - 8 bit MCU      
;* Assembler/
;* Linker       : MPASM
;* Company      : Microchip Technology, Inc.
;*
;*                     Software License Agreement                      
;*                                                                     
;* Microchip Technology Inc. (Microchip) licenses this software to you
;* solely for use with Microchip microcontrollers and Microchip serial
;* EEPROM products.  The software is owned by Microchip and/or its
;* licensors, and is protected under applicable copyright laws.  All
;* rights reserved.
;*
;* SOFTWARE IS PROVIDED "AS IS."  MICROCHIP AND ITS LICENSOR EXPRESSLY
;* DISCLAIM ANY WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED,
;* INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
;* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
;* NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP AND ITS LICENSORS BE
;* LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
;* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
;* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
;* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
;* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
;*
;* Author                   Date                Version         Comment
;*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;* Deepak Kumar Rana        20-Feb-08           V1.0            Original
;*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


;*******************Includes**************************************
 #include    "SRAM_Driver.inc" 				; This includes file where the constants,variables decalred for the SPI

 		 

	CODE
;**************************************************************************
; Function              :Init_SPI
; Precondition          :Make sure that the controller has an on chip MSSP module 
; Side Effect           :Make sure that the SPI mode and the SCK frequecy are as per the specifications of SRAM  
; Overview              :Initializes the MSSP moudle of the controller for SPI mode 1
;                        CKP =0 CKE=1
;**************************************************************************

Init_SPI:

	global Init_SPI    						;Make function available to across files
	
	banksel TRISC     						;select bank of TRISC
	
	bcf     TRISC,2   						;CS  pin output
	bcf     TRISC,3    						;SCK pin output
	bsf     TRISC,4    						;SDI pin input
	bcf     TRISC,5    						;SDO pin output 
	
	banksel PORTC
	bsf      CS        						;Disabling the chip

	banksel SSPCON2    						;initialize the SPI configurataion register
	movlw 0x00
	movwf SSPCON2     						; clearing the flags
	movlw 0x40		  						;cke = 1
	movwf SSPSTAT    						; data tx on falling edge CKE=1, SMP=0 sampling at the middle of data ouptut time
	
	banksel SSPCON
	movlw  0x20								;ckp = 0,enable the port pins for SPI (SSPEN=1)
	movwf SSPCON        					; SCK = Fosc/4, SPI master, SPI 0,1
	
	return


;**************************************************************************
; Function              :Init_Datamem
; Precondition          :The data to be written to Data_Writepage location must be stored in Data_Out
; Side Effect           :We are writing the same data to all the 32 bytes to the location for demo. In your
;						 application modify the routine appropriately
; Overview              :Initializes the data memory location of the controller to store 
;						 the values that need to written to the SRAM. We use indirect addressing
;						 and store the data that in location defined in Data_Writepage
;                  
;**************************************************************************



   Init_Datamem:
   
   global Init_Datamem
   

   movlw  low (Data_Writepage)				; intialize the FSR to the controller data memory
   movwf  FSR
   bankisel Data_Writepage
   
write_ConRam
   banksel Data_Out
   movfw  Data_Out            			    ;data that is being stored
   movwf  INDF               				;write to location pointed by the FSR
   incf   FSR,1								;point to next memory location
   banksel count
   decfsz count,1	     					;check if all data has been written or not
   goto write_ConRam
      
   return

;**************************************************************************
; Function              : SRAMWritePage  
; Precondition          : Configure SRAM for page mode before calling this routine
; Side Effects          : User needs to make sure that he writing within the page  
; Overview              : This performs the operation of writing to a page in the SRAM.
;						  Indirect addressing is used to access the data from location  
;						  Data_Writepage and write to SRAM
;                        
;**************************************************************************

 
   
   SRAMWritePage:
     global SRAMWritePage 
      
    
     movlw  low (Data_Writepage) 			; initialize the memory pointer to the location where we want to read from
     movwf  FSR
     bankisel Data_Writepage    			; make sure the FSR points in the right data bank



    banksel PORTC
    bcf     CS        						; enable SRAM
   
    
    banksel SSPBUF   
    movlw   Write    						; give the write command 0x02
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF						; check for the transmission of the byte
    goto   $-1 

    banksel Addr_High 
    movfw   Addr_High 						;give the high byte of address location
    banksel SSPBUF
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF						;check for the trasmission of the byte
    goto   $-1 
    
    banksel Addr_Low
    movfw   Addr_Low 						;give the low byte of the address location
    banksel SSPBUF 
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 						;check for the trasmission of the byte
    goto   $-1     
   
  Write_SRAM 
    
    movfw   INDF             				; move the contents from data location pointed by FSR
    banksel SSPBUF
    movwf   SSPBUF          				; writet to SSPBUF to start the trasmission 
    banksel SSPSTAT
    btfss   SSPSTAT,BF       				;check if the byte has been trasmitted
    goto    $-1   

    incf    FSR,1            				;increment data memory pointer location
    banksel count
    decfsz  count,1     					;check if all the bytes have been transmitted
    goto   Write_SRAM        				;if not then trasmit the remaining bytes  
   
    banksel PORTC
    bsf      CS 			 				;Disable SRAM

 	return


;**************************************************************************
; Function              : SRAMReadPage  
; Precondition          : SRAM needs to be configured for page mode before calling this routine
; Side Effects          : User needs to make sure that he is reading within the page    
; Overview              : This performs the operation of reading from a page in SRAM.
;						  Indirect addressing is used to copy the data to data memory location  
;						  pointed by Data_Readpage                         
;**************************************************************************
  
 
  SRAMReadPage:
  
  global  SRAMReadPage
   
    movlw  low (Data_Readpage) 				; initialize the memory pointer to the location where we want 
    movwf  FSR                 				; to store the data read from the SRAM
    bankisel  Data_Readpage
  
    banksel PORTC
    bcf     CS        						; enable SRAM
    
    
    banksel SSPBUF
    movlw   Read     						; give the read command 0x3  
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 						; check for the successful transmission of the byte
    goto   $-1 

    banksel Addr_High 
    movfw   Addr_High 						;give the high address location 
    banksel SSPBUF
    movwf   SSPBUF
    banksel SSPSTAT  
    btfss   SSPSTAT,BF 						;check for the successfule transmission of the byte
    goto   $-1 
    
    banksel Addr_Low 						;give the low address location
    movfw   Addr_Low
    banksel SSPBUF 
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 						;check for the succesfull trasmission of the byte
    goto   $-1     
   
  Read_SRAM 
    banksel SSPBUF
    movlw   0xFF     						; do the dumy write to SSBUF to read the SRAM
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 						;check for the succesfull trasmission of the byte
    goto   $-1
    
    banksel SSPBUF
    movfw  SSPBUF     						; copy the read data from SRAM to Work register
    movwf  INDF       						; save the data to the location pointed by FSR
    incf    FSR,1
    banksel count
    decfsz  count,1 						;check if all the data has been read
    goto   Read_SRAM     					;if not then read the remaining bytes
   
    banksel PORTC
    bsf      CS 							;Disable SRAM

 	return
 
;**************************************************************************
; Function              :Seq_Write
; PreCondition          :User needs to make sure that the SRAM has be configured to Sequential mode  
; Side Effects          :Location pointed to by FSR will change at the end of this function.
; Overview              :This performs the operation of writing to memory locations in the SRAM.
;						 Indirect addressing is used to access the data from location  
;						 Data_Writepage and write to SRAM                       
;**************************************************************************
  
 


     SRAMWriteSeq:
     global SRAMWriteSeq
      
    
     movlw  low (Data_Writepage) 			; initialize the memory pointer to the location where we want to read from
     movwf  FSR
     bankisel Data_Writepage

    banksel PORTC
    bcf     CS       						 ; enable SRAM
   
   
    banksel SSPBUF    
    movlw   Write   						 ; give the write command 0x2
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 						 ;check for the succesfull trasmission of the byte
    goto   $-1 

    banksel Addr_High  
    movfw   Addr_High 						 ;give the high byte of the address
    banksel SSPBUF
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF  					 ;check for the succesfull trasmission of the byte
    goto   $-1 
    
    banksel Addr_Low
    movfw   Addr_Low  						 ;give the low byte of the address
    banksel SSPBUF 
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 						 ;check for the succesfull trasmission of the byte
    goto   $-1     
   
  Write_Seq 
    
    movfw   INDF     						 ;read the data from the location pointed by the FSR    
    banksel SSPBUF
    movwf   SSPBUF   						 ;write the data read to the SRAM
    banksel SSPSTAT
    btfss   SSPSTAT,BF 						 ;check for the succesfull trasmission of the byte
    goto   $-1   
    incf    FSR,1
    
    decfsz  count,1 						 ;check if all the data has been trasmitted
    goto   Write_Seq 
   
    banksel PORTC
    bsf      CS 							 ;Disable SRAM

 	return
  
;**************************************************************************
; Function              :Seq_Read
; Precondition          :User needs to make sure that the SRAM has been configured to Sequential mode 
; Side Effects          :Location pointed to by FSR will change at the end of this function 
; Overview              :This performs the operation of writing to SRAM from the controller across the page boundaries
;						 Indirect addressing is used to write the data Read from SRAM to location  
;						 Data_Readpage in controller data memory                       
;**************************************************************************
  
 

   SRAMReadSeq:
  
   global  SRAMReadSeq
   
    movlw  low (Data_Readpage) 					;initialize the memory pointer to the location where we want to read from
    movwf  FSR
    bankisel Data_Readpage
   
    banksel PORTC
    bcf     CS        							;enable SRAM
     
    banksel SSPBUF   
    movlw   Read    							;give the read command to the SRAM 0x3
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 							;check for the succesfull trasmission of the byte
    goto   $-1 

    banksel Addr_High 
    movfw   Addr_High 							;High byte of the address location in SRAM
    banksel SSPBUF
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 							;check for the successful transmission of the byte
    goto   $-1 
    
    banksel Addr_Low
    movfw   Addr_Low  							;Low byte of the address location in SRAM
    banksel SSPBUF 
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 							;check for the succesfull trasmission of the byte
    goto   $-1     
   
  Read_Seq 
    banksel SSPBUF
    movlw   0xFF    							;to read the SRAM location we perform the dummy write to SSPBUF
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF							;check for the succesfull trasmission of the byte
    goto   $-1
    banksel SSPBUF
    movfw  SSPBUF    							;save the data read from SRAM in Controller data memory
    movwf  INDF      						    ;save the data in data memory loation pointed by the FSR 
    incf    FSR,1
    banksel count
    decfsz  count,1 							;check if all the data has been read
    goto   Read_Seq 
   
    banksel PORTC
    bsf      CS 								;Disable SRAM

    return


;**************************************************************************
; Function              :SRAMWriteByte
; Precondition          :Make sure that before calling this fuction the status register of SRAM is configured for Byte Mode 
; Side Effects          :None
; Overview              :This performs the operation of writing byte stored in Data_Out location to SRAM. 
;						                      
;**************************************************************************
  

 
	SRAMWriteByte:

	global SRAMWriteByte
    
    banksel PORTC
    bcf     CS        						    ;enable SRAM
     
    banksel SSPBUF   
    movlw   Write   							;give the write command to the SRAM
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF  						;check for the succesfull trasmission of the byte
    goto   $-1 

    banksel Addr_High
    movfw   Addr_High  							;give the high byte of the address
    banksel SSPBUF
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF							;check for the succesfull trasmission of the byte
    goto   $-1 
    
    banksel Addr_Low
    movfw   Addr_Low  							;give the low byte of the address
    banksel SSPBUF 
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF  						;check for the succesfull trasmission of the byte
    goto   $-1     
    
    banksel Data_Out 
    movfw   Data_Out 							;write the data to be written to SRAM on to the SSPBUF
    banksel SSPBUF
    movwf   SSPBUF    							;start the trasmission
    banksel SSPSTAT
    btfss   SSPSTAT,BF  						;check for the succesfull trasmission of the byte
    goto   $-1     
   
   	banksel PORTC
    bsf     CS   								;disable RAM  
    return


;**************************************************************************
; Function              :SRAMReadByte 
; Precondition          :Make sure that before calling this fuction the status register of SRAM is configured for Byte Mode 
; Side Effects          :After the end of this function Data_In contains the data read from the SRAM
; Overview              :This performs the operation of reading byte from SRAM.The read content is stored at location Data_In 
;						                      
;**************************************************************************
  
;Read a Byte from the location
    SRAMReadByte 
    global SRAMReadByte 
   
    banksel PORTC
    bcf     CS        							;enable SRAM
    
    
    banksel SSPBUF
    movlw   Read    							;give the read command to the SRAM   
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF  						;check for the succesfull trasmission of the byte
    goto   $-1 

    banksel Addr_High
    movfw   Addr_High 							;High byte of the address location to be read
    banksel SSPBUF
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF  						;check for the succesfull trasmission of the byte
    goto   $-1 
    
    banksel  Addr_Low 
    movfw    Addr_Low 							;Low byte of the address location to read 
    banksel  SSPBUF
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 							;check for the succesfull trasmission of the byte 
    goto   $-1     
    
    banksel SSPBUF
    movlw   0xFF    							;perform the dummy write to read the SRAM location
    movwf   SSPBUF  
    banksel SSPSTAT
    btfss   SSPSTAT,BF 							;check for the succesfull reception of the byte
    goto   $-1     
 
    banksel Data_In
    movfw   SSPBUF
    movwf   Data_In
   
    banksel PORTC
    bsf     CS  								 ;disable RAM  
    
    return

;**************************************************************************
; Function              :SRAMWriteStatusReg
; Precondition          :The mode of operation to be writen to status register is passed from main.asm
; Side Effects          :Writing to the status register changes the SRAM operation mode.
; Overview              :This performs the operation of writing to the status register of SRAM 
;						 This fuction is used to configure SRAM for byte,page,sequential mode                     
;**************************************************************************
  
   SRAMWriteStatusReg:
   
   global SRAMWriteStatusReg
    
    banksel PORTC
    bcf     CS        						 ;enable SRAM
  

    banksel SSPBUF
    movlw   WRSR       						 ;issue the write to status register command to SRAM 
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF  					 ;check for the succesfull trasmission of the byte
    goto   $-1

    banksel SSPBUF     
    movfw   mode    						 ;write the mode of operation of SRAM 
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 						 ;check for the succesfull trasmission of the byte
    goto   $-1 

    banksel  PORTC  
    bsf      CS 							 ;Disable SRAM

    return
  
;**************************************************************************
; Function              :SRAMReadStatusReg
; Precondition          :Make sure that the MSSP module is initialized for SPI
; Side Effects          :None 
; Overview              :This performs the operation of reading from the status register of SRAM 
;						 This fuction is used to check the configuration of SRAM .Data_In contain the data read 
;						 from status register                      
;**************************************************************************
  
  SRAMReadStatusReg:
  
    global SRAMReadStatusReg
    
    banksel PORTC
	bcf     CS       							 ; enable SRAM
  
    banksel SSPBUF
    movlw   RDSR      							;issue the read status register command to the SRAM
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF 							;check for the succesfull trasmission of the byte
    goto   $-1
 
	banksel SSPBUF     
    movlw   0xFF     							;dummy write to SSPBUF to read form SRAM status register 
    movwf   SSPBUF
    banksel SSPSTAT
    btfss   SSPSTAT,BF  						;check for the succesfull reception of the byte
    goto   $-1

    banksel SSPBUF
    movfw   SSPBUF   							; copy the content read to the Data_In location of the controller
    movwf   Data_In 
   
    banksel PORTC
    bsf      CS 								;Disable SRAM
   
   
    return


;**************************************************************************
  
    END