
;*********************************************************************
;* FileName     : SRAM_Driver.inc
;* Dependencies : See INCLUDES section below
;* Processor    : PIC16F family of PICmicro - 8 bit MCU      
;* Assembler/
;* Linker       : MPASM
;* Company      : Microchip Technology, Inc.
;*
;*                     Software License Agreement                      
;*                                                                     
;* Microchip Technology Inc. (Microchip) licenses this software to you
;* solely for use with Microchip microcontrollers and Microchip serial
;* EEPROM products.  The software is owned by Microchip and/or its
;* licensors, and is protected under applicable copyright laws.  All
;* rights reserved.
;*
;* SOFTWARE IS PROVIDED "AS IS."  MICROCHIP AND ITS LICENSOR EXPRESSLY
;* DISCLAIM ANY WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED,
;* INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
;* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
;* NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP AND ITS LICENSORS BE
;* LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
;* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
;* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
;* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
;* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
;*
;* Author                   Date                Version         Comment
;*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;* Deepak Kumar Rana        20-Feb-08           V1.0            Original
;*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

 #include "P16F887.inc"

	errorlevel  -302   			;Turn off banking message
    errorlevel  -205   			;Turn off found directive in column 1
   	errorlevel  -207   			;Turn off warnings of found label after column 1

;**************************************************************************
	;Hardware Definations
;**************************************************************************	
	#define  CS    PORTC,2    	;RC2 pin(17) is CS for PIC16F887
	#define  SCK   PORTC,3    	;RC3 pin(18) is SCK
	#define  SDI   PORTC,4    	;RC4 pin(23) is SDI
	#define  SDO   PORTC,5    	;RC5 pin(24) is SDO
	
;**************************************************************************	
	;Define SPI commands
;**************************************************************************	
	#define  Read   0x03     	;read from memory array beginning at the selected address
	#define  Write  0x02     	;write to memory array beginning at the selected address
	#define  RDSR   0x05     	;read the status register
	#define  WRSR   0x01     	;write the status register

;**************************************************************************	
	;Mode selection by settting bit seven and six in Staus register
;**************************************************************************	
	#define Byte_Mode 0x01  	;bit7 =0 bit6 =0 setting LSB 1 disables the hold functionality
	#define Page_Mode 0x81  	;bit7 =1 bit6 =0
	#define Seq_Mode  0x41  	;bit7 =0 bit6 =1  sequential mode
	

;** D E F I N I T I O N S ****************************************

;*******************RAM register definitions**********************
 
    write	UDATA  
    Data_Writepage res 32 		;data to be written SRAM is saved here
    Addr_High      res  1 		;address location where to write/read
    Addr_Low       res  1 		;address location where to write/read
    Data_Out       res  1 		;data to be written to SRAM
    Data_In        res  1 		;data read from SRAM
    count		   res  1 		;number of bytes/location in SRAM to be read/written
    mode           res  1 		;mode of operation for SRAM
   
 ;*******************RAM register definitions********************** 
    read	UDATA  
    Data_Readpage  res 32 		;data read from the SRAM is saved here

 ;**************************************************************************    	
;Allow following variables to access from other files.
   	global	Data_Out
   	global	Data_In
   	global	Addr_High
   	global	Addr_Low
   	global  Data_Writepage
   	global  Data_Readpage
   	global  count
    global  mode
    
   
 ;************************************************************************** 

