

/*********************************************************************
;* FileName     : Main_Demo.c
;* Dependencies : See INCLUDES section below
;* Processor    : PIC18F family of PICmicro - 8 bit MCU      
;* Assembler/
;* Linker       : MPLAB C18 V3.20
;* Company      : Microchip Technology, Inc.
;*
;*                     Software License Agreement                      
;*                                                                     
;* Microchip Technology Inc. (Microchip) licenses this software to you
;* solely for use with Microchip microcontrollers and Microchip serial
;* EEPROM products.  The software is owned by Microchip and/or its
;* licensors, and is protected under applicable copyright laws.  All
;* rights reserved.
;*
;* SOFTWARE IS PROVIDED "AS IS."  MICROCHIP AND ITS LICENSOR EXPRESSLY
;* DISCLAIM ANY WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED,
;* INCLUDING BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
;* MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
;* NON-INFRINGEMENT. IN NO EVENT SHALL MICROCHIP AND ITS LICENSORS BE
;* LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
;* DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
;* PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
;* BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
;* ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
;*
;* Author                   Date                Version         Comment
;*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
;* Deepak Kumar Rana        20-Feb-08           V1.0            Original
;*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
*/

#include <p18F45K20.h>

#include "SRAM_main.h"


#pragma config WDTEN= OFF,BOREN =OFF ,MCLRE =ON,LVP =OFF

#pragma config FOSC = HS,FCMEN =ON,PWRT = ON,HFOFST = OFF    


void main()
{
   /* Declare local variables for the address byte, data and number of byres information to 
      passed to the SRAM read write functions */    
        unsigned char low_addr,high_addr,data,num_byte;
    	Init_SPI();							//call the intialization MSSP  module for SPI	
	 

/* Byte Mode Demo 
   In this demo we will configure the SRAM for Byte mode,and write at a specified address
   location of the SRAM. To reconfirm we will read back the address location.
*/      

  
        SRAMWriteStatusReg(Byte_Mode);		 // Configure SRAM in Byte mode
        SRAMReadStatusReg();				 // Vefiying for the configuration	
        high_addr =0x00;				     // Specifying the high byte of  address location	
        low_addr  =0x20;				     // low byte of the address location
        data      =0x55;                     // data to be written to that location
        SRAMWriteByte(high_addr,low_addr,data); // pass the address and data to be written information   
        SRAMReadByte(high_addr,low_addr);       // read from the specified address and data location
    

/* Page Demo 
   In this Demo we have configured the SRAM to page mode. Each page of SRAM is of 32 bytes. We create a Data_Writepage
   location in the data memory of the MCU and store the 32 bytes in specified locaiton.We write these 32 bytes from the MCU data
   memory to the SRAM page.To verify the write to SRAM we read SRAM and store the content on the Data_Readpage location in the data memory of
   the MCU.
*/        


        SRAMWriteStatusReg(Page_Mode); 		 // Configure SRAM to Page mode
        SRAMReadStatusReg();				 // vefiry the configuratin by reading Status register of SRAM
        num_byte =32;						 // specify the number of bytes to be written
        data  = 0xA4;						 // data to be written to the location in the data memory of controller	
        Init_Datamem(num_byte,data);         // store the data in the specified data memory of the controller
        high_addr =0x00;					 // High byte of the address location of SRAM
        low_addr  =0x20;					 // Low Byte of the address location
        SRAMWritePage(high_addr,low_addr,num_byte);//pass the address and number of bytes to be written 
        num_byte =32;                        // Number of bytes to be read bback from SRAM
        SRAMReadPage(high_addr,low_addr,num_byte);// pass the address and number of bytes to read from the address location 

/* Page Demo End */
    
       
/*Sequence Demo 
  In this Demo we have configured the SRAM for sequence mode. In this mode we can write across the page boundary. For the demo
  we first save the 32 bytes to be written to the SRAM in the MCU data memory location Data_Writepage.We then write these 32 bytes
  to the SRAM. To verify the write to SRAM we read SRAM and store the content on the Data_Readpage location in the data memory of
  the MCU.
*/



       SRAMWriteStatusReg(Seq_Mode);		// configure the SRAM to Sequence mode
       SRAMReadStatusReg(); 				// verify configuration of SRAM by reading the status register
       num_byte =32;						// number of bytes to be written to the Data memory of the controller
       data  =0x33; 						// data to be stored in the data memory
       Init_Datamem(num_byte,data);			// Store the in the data memory of the controller
       high_addr =0x00;						// High byte of the address location of SRAM
       low_addr  =0x10;						// Low byte of the address location of SRAM
       SRAMWriteSeq(high_addr,low_addr,num_byte);// pass the adddress and the number of bytes to be written 
       num_byte =32;						// Number of bytes to be read	
       SRAMReadSeq(high_addr,low_addr,num_byte);// pass the address and number of bytes to read from that address location onwards 

/* Sequence Demo End */

        while(1);	
}
